#include "BStream.h"
#include <stdio.h>


// export the HSF object to a file

static void WriteObject(BBaseOpcodeHandler * handler, BStreamFileToolkit * tk, FILE * fp,
                        char * buffer, int size) 
{
    TK_Status status;

    // ask the toolkit to write out the object; the toolkit will write out as much as
    // the current buffer can hold
    status = handler->Write(*tk);

    // if there is still more data left to write; we have to loop through 
    // generating and writing buffers of data until we're done
    while (status == TK_Pending) 
    {
        // due to compression, preparing a new buffer might immediately fill that buffer
        // so we need an extra check to possibly export that buffer and prepare a new buffer
        while (status == TK_Pending) 
        {
            // ask the toolkit how much of the buffer is currently filled
            // this will normally be the same as the buffer size that we passed in
            int count = tk->CurrentBufferLength();

            // write the current out to our file since this example deals with 
            // exporting a file; however, instead of writing to a file, the buffer of
            // data could be sent to an application data-structure, or transmitted
            // over a network ('streamed'), etc...
            fwrite(buffer, sizeof (char), count, fp);

            // tell the toolkit where it can resume writing data; in this case, we
            // can reuse the same buffer once we're done with it
            status = tk->PrepareBuffer(buffer, size);
        }

        // ask the toolkit to continue trying to write out the object's data
        status = handler->Write(*tk);
    }
}




int main() 
{
    // create a new file toolkit object
    BStreamFileToolkit *mytoolkit = new BStreamFileToolkit;

    char mybuffer[4096];
    int size=4096;
    int count;

    FILE *flptr;

    // open a file called 'testfile.hsf' 
    flptr=fopen("testfile2.hsf", "wb");

    // tell the toolkit where to begin writing data; the size of the 'utility' buffer
    // can be anything; for this example, we use a 4k buffer
    mytoolkit->PrepareBuffer(mybuffer, size);

    BBaseOpcodeHandler *handler;
    TK_Open_Segment *seghandler;
    TK_Point *pointhandler;
    TK_Polypoint *linehandler;
    TK_Camera *viewhandler;
    TK_Color_RGB *colorhandler;
    TK_Visibility *vishandler;

    // create a TK_Header object and export it's associated HSF objects,
    // which include the TKE_Comment and TKE_File_Info objects
    // TK_Header automatically formats the TKE_Comment to contain file version 
    // information.  
    handler = new TK_Header();
    WriteObject(handler, mytoolkit, flptr, mybuffer, size);
    delete handler;

    // create a TK_Camera object to export a TKE_View opcode; this defines an initial
    // camera used to view the scene-graph contained in the file
    viewhandler = new TK_Camera (TKE_View);
    viewhandler->SetView("view0");
    viewhandler->SetProjection(TKO_Camera_Orthographic);
    viewhandler->SetPosition(0.0, 0.0, -5.0);
    viewhandler->SetTarget(0.0, 0.0, 0.0);
    viewhandler->SetUpVector(0.0, 1.0, 0.0);
    viewhandler->SetField(2.0, 2.0);
    WriteObject(viewhandler, mytoolkit, flptr, mybuffer, size);
    delete viewhandler;

    // export a TKE_Start_Compression object
    handler = new TK_Compression(TKE_Start_Compression);
    WriteObject(handler, mytoolkit, flptr, mybuffer, size);
    delete handler;

    // export a TKE_Open_Segment objec; the TK_Open_Segment opcode handler
    // automatically supports this opcode, so it does not need to be passed
    // into the constructor
    seghandler= new TK_Open_Segment;
    seghandler->SetSegment("geometry");
    WriteObject(seghandler, mytoolkit, flptr, mybuffer, size);
    delete seghandler;

    // export a TKE_Polyline object
    float pts[]={1.0, 1.0, 0.0, 1.0, -1.0, 0.0, 0.5, 0.5, 0.0};
    linehandler= new TK_Polypoint(TKE_Polyline);
    linehandler->SetPoints(3, pts);
    WriteObject(linehandler, mytoolkit, flptr, mybuffer, size);
    delete linehandler;

    // export a TKE_Marker object
    pointhandler = new TK_Point(TKE_Marker);
    pointhandler->SetPoint(0.0, 0.0, 0.0);
    WriteObject(pointhandler, mytoolkit, flptr, mybuffer, size);
    delete pointhandler;

    vishandler = new TK_Visibility();
    vishandler->SetGeometry (TKO_Geo_Marker|TKO_Geo_Line);
    vishandler->SetValue (TKO_Geo_Marker|TKO_Geo_Line);
    WriteObject(vishandler, mytoolkit, flptr, mybuffer, size);
    delete vishandler;

    colorhandler = new TK_Color_RGB();
    colorhandler->SetGeometry (TKO_Geo_Marker|TKO_Geo_Line);
    colorhandler->SetRGB(1.0,0.0,0.0);
    WriteObject(colorhandler, mytoolkit, flptr, mybuffer, size);
    delete colorhandler;
	
    // export a TKE_Close_Segment object; the TK_Close_Segment opcode handler
    // automatically supports this opcode, so it does not need to be passed
    // into the constructor
    handler = new TK_Close_Segment;
    WriteObject(handler, mytoolkit, flptr, mybuffer, size);
    delete handler;

    // export a TKE_Stop_Compression object
    handler = new TK_Compression(TKE_Stop_Compression);
    WriteObject(handler, mytoolkit, flptr, mybuffer, size);
    delete handler;

    // export a TKE_Termination object
    handler = new TK_Terminator(TKE_Termination);
    WriteObject(handler, mytoolkit, flptr, mybuffer, size);
    delete handler;

    // ask the toolkit how much of the final buffer is filled
    count = mytoolkit->CurrentBufferLength();

    // write out the final buffer
    fwrite(mybuffer, sizeof (char), count, flptr);

    // close the file
    fclose(flptr);

    // clean up the toolkit object
    delete mytoolkit;

    return 0;
}


